"""
Created on 3 May 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

https://packaging.python.org/guides/single-sourcing-package-version/
"""

__version__ = '3.3.11'


def version():
    return 'SCS Philips Hue ' + __version__
